 LIST P=16F628, R=DEC    ; Use the PIC16F628 and decimal system 

;===============================================================================
;Program: Opakovac pro PMR pasmo.
;Stanice: Intek 2000, nerozsirena, mirne upravena.
;Autor: Jan Matuska, walda@starhill.org, toho casu FSI VUT v Brne
;Datum: 9.5.2004
;===============================================================================

 #include "P16F628.INC"  ; Include header file 

; Config pro zapojeni bez krystalu a bez externiho MCLR 
 __config  _INTRC_OSC_NOCLKOUT &_LVP_OFF & _WDT_ON & _PWRTE_ON & _BODEN_OFF & _MCLRE_OFF 


;--- definice promennych ---
pau         equ 20H     ;cekaci smycky
count       equ 21H     ;cekaci smycky
cnt1        equ 22H     ;timeout
cnt2        equ 23H     ;timeout
cnt3        equ 24H     ;timeouty
no1         equ 25H     ;prijaty DTMF kod, rad 1
no2         equ 26H     ;prijaty DTMF kod, rad 10
no3         equ 27H     ;prijaky DTMF kod, rad 100
no4         equ 28H     ;prijaty DTMF kod, rad 1000
rectime     equ 29H     ;delka nahravky v VM800
straddr     equ 2AH     ;servisni menu, adresa EEPROM
chci        equ 2BH     ;krokovani ctcss a kanal
akt         equ 2CH     ;krokovani ctcss a kanal
temp        equ 2DH     ;pomocna
temp2       equ 2EH     ;pomocna
temp3       equ 2FH     ;pomocna
mtemp       equ 30H     ;generator morseovky cisel
mznak       equ 31H     ;generator morseovky cisel
recflag     equ 32H     ;prekroceni urcite doby nahravky


;--- nastaveni jmen pro vstupy a vystupy ---
; vystupy z PICu
modekey     equ 0H      ;tlacitko MODE pres 4066
upkey       equ 06H     ;tlacitko UP pres 4066
vmplay      equ 02H     ;NEG. vyvod prehravani VM888
vmrec       equ 03H     ;NEG. vyvod zaznamu VM888
ptt         equ 04H     ;tlacitko PTT
beep        equ 07H     ;pin zvukoveho vystupu
; vstupy do PICu
std         equ 04H     ;vstup signalizujici prijem DTMP tonu
ctcin       equ 05H     ;NEG. vstup z MCU Inteka, oznamuje prijem s CTCSS kodem



;--- nastaveni konstatnt ---
  ;servisni kod ????, ten dalkove nesmi jit zmenit               
  constant service1=?
  constant service2=?
  constant service3=?
  constant service4=?

  ;domovsky kanal
  constant domkanal=?
  ;ctcss pri broadcast rignu a automatovi
  constant domctcss=?

  ;adresa v EEPROM s aktualnim kanalem
  constant adrakt=010h

  ;adresa v EEPROM s aktualnim ctcss kodem
  constant adrctcss=011h   

  ;delka nahravani pro modul VM888, krokovani neni presne po 100ms
  constant maxrec=165



;--- nastaveni dat v EEPROM ---
	org	0x2100		
  ;Nasledujci cisla se daji dalkove menit, toto jsou vychozi po naprogramovani
  ;cislo pro zapnuti repeateru
	de	?,?,?,?
  ;cislo pro broadcast ring s CTCSS
  de  ?,?,?,?
  ;cislo pro test pip
  de  ?,?,?
  
	org	0x2110		
  ;aktualni stavy kanalu a CTCSS, tady nemenit, vychazi to z konstant
  de domkanal, 0



;************************     Reset procesoru     *************************

              ORG 0
              goto start

              ORG 4
              retfie 


; ##############################################################################
; Zacatek programu a inicializace ----------------------------------------------
start         bcf STATUS,RP0      ;bank 0
              movlw 7             ;7 do W
              movwf CMCON         ;CMCON=7 set comparators off   
              bsf STATUS,RP0      ;bank 1
	            movlw 0h            ;0 do W
              movwf VRCON         ;W do VRCON
              ; --- Nastaveni portu ---
              bsf STATUS,RP0      ;bank 1
              bcf OPTION_REG, 7   ;enable internal pullups
              movlw b'00110000'   ;port A 1=in, 0=out
              movwf TRISA         ;portA
              movlw b'01101111'   ;input is all, port B pins 2,4,5 and 6 are OUT 
              movwf TRISB         ;portB
              ; --- Inicializace ---
              bcf STATUS,RP0      ;bank 0
              bsf PORTA,vmplay    ;vypne prehravani
              bsf PORTA,vmrec     ;vypne nahravani
              bcf PORTA,modekey   ;uvolni tlacitko MODE
              bcf PORTA,upkey     ;uvolni tlacitko UP     
 		          clrf cnt1           ;smaze cnt1
		          clrf cnt2           ;smaze cnt2
		          clrf no1            ;smaze no1
		          clrf no2	          ;smaze no2
		          clrf no3            ;smaze no3
		          clrf no4            ;smaze no4
              clrf PORTB          ;smaze PORTB
              movlw 5             ;5 do W
              call MorseCode      ;lokalne odpipa morseovku, jen pro test
              call Delay1Sec      ;pocka 1 sekundu, tak aby se stanice zapla
              movlw 5             ;5 do W
              call MorseCode      ;lokalne odpipa morseovku, jen pro test
              call Zakempuj       ;nastavi domovsky kanal a vypne ctcss kod
              goto MainLoop       ;skoci na hlavni smycku programu



; ##############################################################################
; MORSE CODE NUMERIC GENERATOR, tabulka musi byt vepredu
; ##############################################################################
MorseCode     call MorseGet       ;zavola tabulku MorseGet, parametr je ve W
              movwf mznak         ;navrat ve W se vlozi do mznak       
              movlw 5             ;delka znaku je 5, do W
              movwf mtemp         ;delka se ulozi do mtemp
MorseLoop     btfss mznak,7       ;je 7my bit log.1?
              goto MorseLoop1     ;ne neni, skok MorseLoop1
              call Beep9          ;ano je, pipne carku
              goto MorseLoop2     ;skok na MorseLoop2
MorseLoop1    call Beep8          ;pipne tecku
MorseLoop2    call Delay100ms     ;pauza 100ms
              rlf mznak,f         ;rotace doleva
              decfsz mtemp,f      ;dekrementace mtemp, pokud je mtemp=0,preskoci
              goto MorseLoop      ;skok na MorseLoop
              return              ;odpipano, navrat

MorseGet      addwf PCL, f        ; <7;3> bit zleva, tecka=0 a carka=1
              retlw B'11111000'   ; 0 
              retlw B'01111000'   ; 1 
              retlw B'00111000'   ; 2 
              retlw B'00011000'   ; 3 
              retlw B'00001000'   ; 4 
              retlw B'00000000'   ; 5 
              retlw B'10000000'   ; 6 
              retlw B'11000000'   ; 7 
              retlw B'11100000'   ; 8 
              retlw B'11110000'   ; 9



; ##############################################################################
; HLAVNI SMYCKAa
; ##############################################################################
MainLoop	   	btfsc PORTA, std	  ;je na vstupu STD log.1?
  		        goto StdGoes        ;ano, na vstupu STD je log.1
		          call MainCNT	      ;zavola MainCNT
		          goto MainLoop       ;konec hlavni smycky, skok na MainLoop
MainCNT		    call Pause          ;pauza
              incf cnt1		        ;inkrementace cnt1
		          movlw 255           ;255 do W
		          subwf cnt1,W        ;odecte cnt1 od W a vysledek do W
		          btfss STATUS,Z      ;je cnt=255?
		          return              ;neni, navrat
              clrf cnt1           ;ano je, smaze cnt1
              incf cnt2           ;inkrementuje cnt2
		          movlw 15            ;15 do W
		          subwf cnt2,W        ;odecte cnt2 od W a vysledek do W
		          btfss STATUS,Z      ;je cnt=15?
		          return              ;neni, navrat
 		          movlw 0FFh          ;ano je, 255 do W
		          movwf no1           ;Wcko do no1, tedy smaze DTMF kod a predesly stream
		          return              ;navrat

; po prichodu STD , neni to podprogram
StdGoes       clrf cnt1           ;smaze cnt1
              clrf cnt2           ;smaze cnt2
              movfw no3           ;no3 do W
		          movwf no4           ;W do no4
              movfw no2           ;no2 do W
		          movwf no3           ;W do no3
		          movfw no1           ;no1 do W
		          movwf no2           ;W do no2
              call MT2No1         ;hodnota z MT8870 do no1
              call Beep1          ;pipne, jen pro info !!
StdWait		    clrwdt              ;proti uchylum, kteri by drzeli ton dele jak 2.3sec
              btfsc PORTA,std     ;je signal STD stale v log.1?
		          goto StdWait        ;ano je, skok na StdWait
;zde je test servisni cisla
CheckServNo   movlw service1      ;konstanta service1 do W
  	         	subwf no4,W         ;odecte no4 od W a vysledek do W
		          btfss STATUS,Z      ;je no4=service1?
		          goto CheckRptrNo    ;neni, skok na test dalsiho cisla
              movlw service2      ;konstanta service2 do W
  	         	subwf no3,W         ;odecte no3 od W a vysledek do W
		          btfss STATUS,Z      ;je no3 service2?
		          goto CheckRptrNo    ;neni, skok na test dalsiho cisla
              movlw service3      ;konstanta service3 do W
  	         	subwf no2,W         ;odecte no2 od W a vysledek do W
		          btfss STATUS,Z      ;je no2=service3?
		          goto CheckRptrNo    ;neni, skok na test dalsiho cisla
              movlw service4      ;konstanta service1 do W
  	         	subwf no1,W         ;odecte no1 od W a vysledek do W
		          btfss STATUS,Z      ;je no1=service4?
		          goto CheckRptrNo    ;neni, skok na test dalsiho cisla
              goto ServiceMode    ;sekvence vyhovela, skok na funkci ServiceMode
;zde je test cisla na rezim opakovac
CheckRptrNo   movlw 0             ;adresa 0 v EEPROM
              call ReadEE         ;precte EEPROM do W
  	         	subwf no4,W         ;odecte no4 od W, vysledek do W
		          btfss STATUS,Z      ;je no4=W?
		          goto CheckBrcsNo    ;neni, skok na test dalsiho cisla
              movlw 1             ;adresa 1 v EEPROM
              call ReadEE         ;precte EEPROM do W
  	         	subwf no3,W         ;odecte no3 od W, vysledek do W
		          btfss STATUS,Z      ;je no3=W?
		          goto CheckBrcsNo    ;neni, skok na test dalsiho cisla
              movlw 2             ;adresa 2 v EEPROM
              call ReadEE         ;precte EEPROM do W
  	         	subwf no2,W         ;odecte no2 od W, vysledek do W
		          btfss STATUS,Z      ;je no2=W?
		          goto CheckBrcsNo    ;neni, skok na test dalsiho cisla
              movlw 3             ;adresa 3 v EEPROM
              call ReadEE         ;precte EEPROM do W
  	         	subwf no1,W         ;odecte no1 od W, vysledek do W
		          btfss STATUS,Z      ;je no1=W?
		          goto CheckBrcsNo    ;neni, skok na test dalsiho cisla
              goto RepeaterMode   ;sekvence vyhovela, skok na funkci RepeaterMode
;zde je test cisla na broadcast rezim
CheckBrcsNo   movlw 4             ;adresa 4 v EEPROM
              call ReadEE         ;precte EEPROM do W
  	         	subwf no4,W         ;odecte no4 od W, vysledek do W
		          btfss STATUS,Z      ;je no4=W?
		          goto CheckTestNo    ;neni, skok na test dalsiho cisla
              movlw 5             ;adresa 5 v EEPROM
              call ReadEE         ;precte EEPROM do W
  	         	subwf no3,W         ;odecte no3 od W, vysledek do W
		          btfss STATUS,Z      ;je no3=W?
		          goto CheckTestNo    ;neni, skok na test dalsiho cisla
              movlw 6             ;adresa 6 v EEPROM
              call ReadEE         ;precte EEPROM do W
  	         	subwf no2,W         ;odecte no2 od W, vysledek do W
		          btfss STATUS,Z      ;je no2=W?
		          goto CheckTestNo    ;neni, skok na test dalsiho cisla
              movlw 7             ;adresa 7 v EEPROM
              call ReadEE         ;precte EEPROM do W
  	         	subwf no1,W         ;odecte no1 od W, vysledek do W
		          btfss STATUS,Z      ;je no1=W?
		          goto CheckTestNo    ;neni, skok na test dalsiho cisla
              goto BroadMain      ;sekvence vyhovela, skok na funkci Broadcast
;zde je test cisla na test rezim
CheckTestNo   movlw 8             ;adresa 8 v EEPROM
              call ReadEE         ;precte EEPROM do W
  	         	subwf no3,W         ;odecte no4 od W, vysledek do W
		          btfss STATUS,Z      ;je no4=W?
		          goto StdGoesBad     ;neni, skok na StdGoesBad
              movlw 9             ;adresa 9 v EEPROM
              call ReadEE         ;precte EEPROM do W
  	         	subwf no2,W         ;odecte no2 od W, vysledek do W
		          btfss STATUS,Z      ;je no2=W?
		          goto StdGoesBad     ;neni, skok na StdGoesBad
              movlw 10            ;adresa 10 v EEPROM
              call ReadEE         ;precte EEPROM do W
  	         	subwf no1,W         ;odecte no1 od W, vysledek do W
		          btfss STATUS,Z      ;je no1=W?
		          goto StdGoesBad     ;neni, skok na StdGoesBad
              goto TestMode       ;sekvence vyhovela, skok na funkci Test
StdGoesBad    goto MainLoop       ;navrat do hlavni smycky programu




; ##############################################################################
; funkce REPEATER - tak tady funguje repeater mode...
; ##############################################################################
RepeaterMode  bsf PORTB,ptt       ;zaklicuje
              call Delay1Sec      ;pocka 1 sekundu
              call BeepRepeater   ;zapipa repeater znelku
              call Delay100ms     ;pocka 100ms
              bcf PORTB,ptt       ;odklicuje
              clrf cnt1           ;smaze cnt1
              clrf cnt2           ;smaze cnt2
              clrf rectime        ;smaze rectime
RptrLoop      btfsc PORTA, std	  ;je signal STD v log.1?
  		        goto RptrStdGoes    ;ano je, skok na RptrStdGoes
              call Pause          ;pauza
              call Pause          ;pauza
       		    incf cnt1		        ;inkrementace cnt1
		          movlw 255           ;255 do W
		          subwf cnt1,W        ;odecte cnt1 od W, vysledek do W
		          btfss STATUS,Z      ;je cnt1=255?
		          goto RptrLoop       ;neni, skok na RptrLoop
		          clrf cnt1           ;ano je, smazani cnt1
		          incf cnt2           ;inkrementace cnt2
		          movlw 255           ;255 do W
		          subwf cnt2,W        ;odecte cnt2 od W, vysledek do W
		          btfss STATUS,Z      ;je cnt2=255?
		          goto RptrLoop       ;neni, skok na RptrLoop
              call Beep5          ;lokalni pipnuti na test
              call Beep4
              call Beep1
              call Zakempuj       ;vrati se na domaci kanal a ctcss vypne
              goto MainLoop       ;navratovy skok do hlavni smycky, repeater exit

;po prichodu std v rezimu repeater
RptrStdGoes   clrf cnt1           ;smaze cnt1
              clrf cnt2           ;smaze cnt2
              call MT2No1         ;vystup z MT8870 ulozi do no1
              call Beep1          ;lokalni pipnuti
RptrStdWait	  clrwdt              ;smaze watchdog, proti uchylum, kteri by drzeli ton dele jak 2.3sec
              btfsc PORTA,std     ;je STD signal stale v log.1?
		          goto RptrStdWait    ;ano je, skok na RptrStdWait
              movlw 10            ;dtmf(0) - Repater Exit
  	         	subwf no1,W         ;odecte no1 od W a vysledek do W
		          btfsc STATUS,Z      ;je no1=dtmf(0)?
		          goto RptrExit       ;ano je, skok na ukonceni rezimu opakovac
              movlw 1             ;dtmf(1) - Presun po kanalech
  	         	subwf no1,W         ;odecte no1 od W a vysledek do W
		          btfsc STATUS,Z      ;je no1=dtmf(1)?
		          goto RptrMoveCh     ;ano je, skok na posun po kanalech
              movlw 2             ;dtmf(2) - Rezim automat
   	         	subwf no1,W         ;odecte no1 od W a vysledek do W
		          btfsc STATUS,Z      ;je no1=dtmf(1)?
		          goto RptrAutomat    ;ano je, skok na rezim automat
              movlw 11            ;dtmf(*) - Record
  	         	subwf no1,W         ;odecte no1 od W a vysledek do W
		          btfsc STATUS,Z      ;je no1=dtmf(1)?
		          goto RptrRec        ;ano je, skok na rezim zaznamu
              movlw 12            ;dtmf(#) - Play
  	         	subwf no1,W         ;odecte no1 od W a vysledek do W
		          btfsc STATUS,Z      ;je no1=dtmf(1)?
		          goto RptrPlay       ;ano je, skok na rezim prehravani
              goto RptrLoop       ;navrat do hlavni smycky rezimu repeater

;ukonceni modu repeater           
RptrExit      bsf PORTB,ptt       ;zaklicuje
              call Delay1Sec      ;pocka 1 sekundu
              call Beep5          ;pipnuti do eteru
              call Beep4
              call Beep1
              bcf PORTB,ptt       ;odklicuje
              call Zakempuj       ;vrati se na domaci kanal a ctcss vypne
              goto MainLoop       ;navrat na hlavni smycku, repeater exit     

;nahravani
RptrRec       clrf rectime        ;smaze rectime
              clrf recflag        ;smaze recflag
              movlw maxrec        ;konstantu maxrec do W
              movwf temp          ;W do temp
              bcf PORTA,vmrec     ;vm888 zacne nahravani
RptrRecLoop   call Delay100ms     ;pauza 100ms
              incf rectime        ;inkrementace rectime      
              movlw 3             ;tohle je priznak zpravy delsi jak 300ms
              subwf rectime,W     ;odecte rectime od W, vysledek do W
              btfsc STATUS,Z      ;je rectime=3?
              incf recflag        ;do recflagu se nastavi jednicka
              btfss PORTA,std     ;je signal STD na log.1?
              goto RptrRecLoop1   ;neni, skok na RptrRecLoop1
              call MT2No1         ;ano je, prevod z MT8870 do no1
              movlw 12            ;dtmf(#) - Play, ukonceni zaznamu a start prehravani
  	         	subwf no1,W         ;odecte no1 od W, vysledek do W
		          btfsc STATUS,Z      ;je prichozi dtmf kod roven #?
		          goto RptrRecEnd     ;ano je, skok na RptrRecEnd
RptrRecLoop1  decfsz temp,f       ;dekrementace temp a preskok, je-li temp=0
              goto RptrRecLoop    ;smycka na RptrRecLoop
RptrRecEnd    bsf PORTA,vmrec     ;vm888 ukonci prehravani
              movf recflag,f      ;presun recflag pres ALU, ovlivni Z-bit
              btfsc STATUS,Z      ;je recflag=0?
              goto RptrRecExit    ;ano je, zprava je kratsi nez 300ms
              decf rectime        ;ne neni, odecte 300ms, tj odmazani # na ukonceni recordu
              decf rectime        
              decf rectime
RptrRecExit   goto RptrLoop       ;navrat do hlavni smycky rezimu Repeater

;prehravani
RptrPlay      bsf PORTB,ptt       ;zaklicuje
              call Delay1Sec      ;pocka 1 sekundu
              bcf PORTA,vmplay    ;vm888 zacne prehravani
              movfw rectime       ;rectime do W
              movwf temp          ;W do temp
              incf temp           ;inkrementace temp, kdyby rectime bylo nula
RptrPlayLoop  call Delay100ms     ;pocka 100ms
              decfsz temp,f       ;dekrementace temp a preskok kdyz je temp nula
              goto RptrPlayLoop   ;skok na RptrPlayLoop
              bsf PORTA,vmplay    ;vm888 ukonci prehravani          
              call Beep7          ;roger beep
              bcf PORTB,ptt       ;odklicuje
              goto RptrLoop       ;navrat na hlavni smycku rezimu repeater

;choditko po kanalech
RptrMoveCh    btfsc PORTA, std	  ;je signal STD v log.1?
  		        goto RptrMoveStd    ;ano je, skok na RptrMoveStd
              call Pause          ;pauza
       		    incf cnt1		        ;inkrementace cnt1
		          movlw 255           ;255 do W
		          subwf cnt1,W        ;odecte cnt1 od W, vysledek do W
		          btfss STATUS,Z      ;je cnt1=255?
		          goto RptrMoveCh     ;ne neni, skok na RptrMoveCh
		          clrf cnt1           ;smaze cnt1
		          incf cnt2           ;inkrementace cnt2
		          movlw 15            ;15 do W
		          subwf cnt2,W        ;odecte cnt2 od W, vysledek do W
		          btfss STATUS,Z      ;je cnt2=15?
		          goto RptrMoveCh     ;ne neni, skok na RptrMoveCh
              call Beep4          ;lokalni beep
              clrf cnt1           ;smaze cnt1
              clrf cnt2           ;smaze cnt2
              goto RptrLoop       ;timeout pri zadavani kanalu, navrat do RptrLoop
RptrMoveStd   clrwdt              ;smaze watchdog, ochrana proti tonu delsimu jak 2.3sec
              btfsc PORTA,std     ;je signal STD v log.1?
		          goto RptrMoveStd    ;ano je, skok na RptrMoveStd
              call MT2No1         ;ne neni, prevod z MT8870 do no1
              movlw 1             ;ZACATEK testovacniho INTERVALU do W
              movwf temp          ;W do temp
RptrMoveTest  movfw temp          ;temp do W
              subwf no1,W         ;odecte W od chci
              btfsc STATUS,Z      ;je cislo chci shodne s temp?
              goto RptrMoveIt     ;ano, cislo chci patri do intervalu <1;8>
              movlw 8             ;8 do W  toto je nastaveni KONCE INTERVALU
              subwf temp,W        ;odecte W od 8
              btfsc STATUS,Z      ;nastal uz konec intervalu?
              goto RptrMoveEnd    ;ano, cislo chci se neveslo do intervalu <1;8>
              incf temp           ;nenastal -> inkrementace temp
              goto RptrMoveTest   ;dalsi smycka testu
RptrMoveIt    bsf PORTB,ptt       ;zaklicuje
              call Delay1Sec      ;pocka 1 sec
              movfw no1           ;no1 do W
              call MorseCode      ;odpipa kanal, kam se pujde
              bcf PORTB,ptt       ;odklicuje
              movfw no1           ;no1 do W
              call ChannelSet     ;presune kanal
              bsf PORTB,ptt       ;zaklicuje
              call Delay1Sec      ;pocka 1 sekundu
              call Beep5          ;pipne do eteru
              bcf PORTB,ptt       ;odklicuje
RptrMoveEnd   clrf cnt1           ;smaze cnt1
              clrf cnt2           ;smaze cnt2
              goto RptrLoop       ;navrat do RptrLoop

;automaticky repeater
RptrAutomat   bsf PORTB,ptt       ;zaklicuje
              call Delay1Sec      ;pocka 1 sekundu
              call Beep9          ;morse C
              call Delay100ms
              call Beep8
              call Delay100ms
              call Beep9
              call Delay100ms
              call Beep8
              call Delay300ms     ;pausa mezi znaky
              call Beep9          ;morse T
              call Delay300ms     ;pausa mezi znaky
              call Beep9          ;C
              call Delay100ms
              call Beep8
              call Delay100ms
              call Beep9
              call Delay100ms
              call Beep8
              call Delay300ms     ;pauza za poslednim znakem
              bcf PORTB,ptt       ;odklicuje
              movlw domctcss      ;domctcss do W
              call CtcssSet       ;nastavi CTCSS kod, ktery je ve W
              bsf PORTB,ptt       ;zaklicuje
              call Delay1Sec      ;pocka 1 sekundu
              call Beep2          ;oznameni, ze je automaticky mod aktivni
              call Beep2
              bcf PORTB,ptt       ;odklicuje
              call Delay300ms     ;pocka 300ms

              clrf cnt1           ;smaze cnt1
              clrf cnt2           ;samze cnt2
RptrAutLoop   btfss PORTB, ctcin	;je vstup ctcin na log.0? Tedy slysi Intek signal s ctcss kodem?
  		        goto RptrAutCtcIn   ;ano je
              call Pause          ;ne neni, pauza kvuli timeoutu automatickeho rezimu
       		    incf cnt1		        ;inkrementace cnt1
		          movlw 255           ;255 do W
		          subwf cnt1,W        ;odecte cnt1 od W a ulozi do W
		          btfss STATUS,Z      ;je obsah cnt1 roven W? Tedy je cnt1=255?
		          goto RptrAutLoop    ;ne neni, proto skoc na RptrAutLoop
		          clrf cnt1           ;ano je, smaze cnt1
		          incf cnt2           ;inkrementace cnt2
		          movlw 180           ;90 do W
		          subwf cnt2,W        ;odecte cnt2 od W a ulozi do W 
		          btfss STATUS,Z      ;je obsah cnt2 roven W? Tedy je cnt=90?
		          goto RptrAutLoop    ;ne neni, proto skoc na RptrAutLoop
RptrAutLoop2  bsf PORTB,ptt       ;ano je, timeout vyprsel, zaklicuje
              call Delay1Sec      ;pocka 1 sekundu
              call Beep5          ;ton oznameni ukonceni automatickeho repeateru
              call Beep7
              call Beep1
              bcf PORTB,ptt       ;odklicuje
              movlw 0             ;0 do W, znamena to vypnuty kod CTCSS
              call CtcssSet       ;nastavi CTCSS kod, ktery je ve W
              clrf rectime        ;smaze casovy index delky nahravky
              goto RptrLoop       ;navrat do hlavni smycky klasickeho repeater modu

RptrAutCtcIn  call Delay100ms     ;pauza 100ms
              btfsc PORTB, ctcin  ;je opravdu ctcin v log.0, tedy slysi Intek signal s ctcss kodem?
              goto RptrAutLoop    ;ne neni, zrejme nejake ruseni, navrat do RptrAutLoop
              clrf cnt1           ;aktivita opakovace, smaze cnt1
              clrf cnt2           ;smaze cnt2
              clrf rectime        ;smaze delku nahravky
              clrf recflag        ;smaze recflag
              bcf PORTA,vmrec     ;vm888 zacne nahravat
RptrAutRec    call Delay100ms     ;pauza 100ms
              movlw maxrec        ;maximlani delka vzkazu do W
              subwf rectime,W     ;odecte W od rectime, vysledek do W
              btfss STATUS,Z      ;byla dosazena maximalni delka vzkazu?
              incf rectime        ;nebyla a proto inkrementace rectime
              btfsc PORTA,std     ;prisel behem nahravani STD, tzn prichozi DTMF?
              goto RptrAutStd     ;ano, skok na RptrAutStd
              movlw 8             ;tohle je priznak zpravy delsi jak 800ms
              subwf rectime,W     ;odecte rectime od W, vysledek do W
              btfsc STATUS,Z      ;je rectime=8?
              incf recflag        ;do recflagu se nastavi jednicka
              btfss PORTB, ctcin  ;je vstup ctcin v log.1? Tedy ukonceni relace.
              goto RptrAutRec     ;neni, navrat na zacatek smycky RptrAutRec
              call Pause          ;pauza
              btfss PORTB, ctcin  ;je skutecne na ctcin log.1? Tedy skutecne relace skoncila?
              goto RptrAutRec     ;nene, zrejme ruseni, navrat na RptrAutRec
              bsf PORTA,vmrec     ;vm888 prestane nahravat
              movf recflag,f      ;presun recflag pres ALU, ovlivni Z-bit
              btfsc STATUS,Z      ;je recflag=0?
              goto RptrAutLoop    ;ano je, zprava je kratsi nez 800ms, navrat do RptrAutLoop
              movlw 8             ;8 do W, musi se odecist, protoze Intek zavira CTCSS pozdeji.
              subwf rectime,f     ;obsah W se odecte od rectime, tedy odebere se asi 800ms zpravy, prevazne sumu
              bsf PORTB,ptt       ;zaklicuje
              call Delay1Sec      ;pocka 1 sekundu
              bcf PORTA,vmplay    ;vm888 zacne prehravat
RecAutPlay    call Delay100ms     ;pauza 100ms
              decfsz rectime,f    ;dekrementace rectime a skok jestli je rectime uz 0
              goto RecAutPlay     ;rectime>0, pokracuje prehravani                    
              call Beep7          ;pipne do eteru
              bsf PORTA,vmplay    ;vm888 prestane prehravat
              bcf PORTB,ptt       ;odklicuje
              goto RptrAutLoop    ;skoci do hlavni smycky automatickeho repeateru

RptrAutStd    call MT2No1         ;prevod ze vstupu MT8870 do regostri no1
              bsf PORTA,vmrec     ;vm888 prestane nahravat
              clrf rectime        ;smaze se rectime
              clrf recflag
RptrAutStd2   clrwdt              ;smaze watchdog, proti uchylum, kteri by drzeli ton dele jak 2.4sec
              btfsc PORTA,std     ;je STD stale log.1?
              goto RptrAutStd     ;ano, stale je log.1, skok na RptrAutStd
              movlw 10            ;dtmf(0), 10 do W
  	         	subwf no1,W         ;odecteni no1 od W, vysledek do W
		          btfsc STATUS,Z      ;je no1=dtmf(0)?
              goto RptrAutLoop2   ;ano je, skok na cast, ktera ukoncuje rezim automatickeho repeateru
              bcf PORTA,vmrec     ;vm888 zacne nahravat, ale od zacatku
              goto RptrAutRec     ;ne neni







; ##############################################################################
; BROADCAST MESSAGE S CTCSS
; ##############################################################################
BroadMain     bsf PORTB,ptt       ;zaklicuje
              call Delay1Sec      ;pocka 1 sec
              call Beep5          ;pipne do eteru
              bcf PORTB,ptt       ;odklicuje
              movlw domctcss      ;nastavi domovsky CTCSS kod, domctcss do W
              call CtcssSet       ;provede nastaveni CTCSS podle obsahu W
              bsf PORTB,ptt       ;zaklicuje
              call Delay1Sec      ;pocka 1 sec
              call BroadCQ        ;zapipa CQ
              call BroadCQ        ;zapipa CQ
              call BroadCQ        ;zapipa CQ
              bcf PORTB,ptt       ;odklicuje
              call Zakempuj       ;nastavi domovsky kanal a vypne CTCSS
              clrf no1            ;smaze obsah no1
              goto MainLoop       ;navrat do glavni smycky
BroadCQ       call Beep9          ;C
              call Delay100ms
              call Beep8          
              call Delay100ms
              call Beep9          
              call Delay100ms
              call Beep8          
              call Delay300ms     ;pauza mezi znaky
              call Beep9          ;Q
              call Delay100ms
              call Beep9
              call Delay100ms
              call Beep8
              call Delay100ms
              call Beep2
              call Delay300ms     ;pauza mezi znaky
              return              ;navrat



; ########################################################################
; Mod testu dostupnosti opakovace - jen pipne do eteru
; ########################################################################
TestMode      bsf PORTB,ptt       ;zaklicuje
              call Delay1Sec      ;pocka 1 sec
              call Beep1          ;pipne do eteru
              call Delay100ms     ;pocka 100ms
              call Beep1          ;pipne do eteru
              call Delay100ms     ;pocka 100ms
              bcf PORTB,ptt       ;odklicuje
              goto MainLoop       ;navrat do hlavni smycky MainLoop




; ########################################################################
; Servisni rezim - umoznuje menit kody a hejbat kanalama a ctcss u stanice
; ########################################################################
ServiceMode   bsf PORTB,ptt       ;zaklicuje
              call Delay1Sec      ;pocka 1 sekudnu
              call Beep6          ;pipne servisni melodii
              call Beep6          
              call Beep6          
              call Delay100ms     ;pocka 100ms
              bcf PORTB,ptt       ;odklicuje
              clrf cnt1           ;smaze cnt1
              clrf cnt2           ;smaze cnt2
ServiceLoop   btfsc PORTA, std	  ;je signal STD log.1?
  		        goto ServStdGoes    ;ano je, skok na ServStdGoes
              call Pause          ;pauza
       		    incf cnt1		        ;inkrementace cnt1
		          movlw 255           ;255 do W
		          subwf cnt1,W        ;odecte cnt1 od W, vysledek do W
		          btfss STATUS,Z      ;je cnt1=255?
		          goto ServiceLoop    ;ne neni, skok na ServiceLoop
		          clrf cnt1           ;ano je, smazani cnt1
		          incf cnt2           ;inkrementace cnt2
		          movlw 30            ;30 do W
		          subwf cnt2,W        ;odecte cnt2 od W, vysledek do W
		          btfss STATUS,Z      ;je cnt2=30?
		          goto ServiceLoop    ;ne neni, skok na ServiceLoop
              bsf PORTB,ptt       ;timeout, zaklicuje
              call Delay1Sec      ;pocka 1 sec
              call Beep5          ;znelka oznamujici konec servisniho rezimu
              call Beep7
              call Beep1
              bcf PORTB,ptt       ;odklicuje
              goto MainLoop       ;navrat do hlavni smycky MainLoop
;po prichodu STD v servisnim rezimu
ServStdGoes   clrf cnt1           ;smaze cnt1
              clrf cnt2           ;smaze cnt2
              call MT2No1         ;prevede vystup z MT8870 do no1
              call Beep1          ;lokalni pip
ServStdWait	  clrwdt              ;smaze watchdog, proti uchylum, kteri by drzeli ton dele jak 2.4sec
              btfsc PORTA,std     ;je signal STD ve stavu log.1?
		          goto ServStdWait    ;ano je, skok na ServStdWait
              movlw 11            ;dtmf(*) - ChannelUP
  	         	subwf no1,W         ;odecte no1 od W, vysledek do W
		          btfsc STATUS,Z      ;je no1=dtmf(*)
		          call ServChUp       ;ano je, zavolani skoku kanalu o jeden nahoru
              movlw 12            ;dtmf(#) - CtcssUP
  	         	subwf no1,W         ;odecte no1 od W, vysledek do W
		          btfsc STATUS,Z      ;je no1=dtmf(#)?
		          call ServCtcssUp    ;ano je, zavolani skoku CTCSS kodu o jeden nahoru
              movlw 10            ;dtmf(0) - Sekvence novych kodu do EEPROM
  	         	subwf no1,W         ;odecte no1 do W, vysledek do W
		          btfsc STATUS,Z      ;je no1=dtmf(0)
		          call ServStream     ;ano je, zavolani rutiny pro zadani novych kodu
              goto ServiceLoop    ;navrat do smycky servisniho menu        
                                 
ServChUp      call PressMode      ;kliknuti na tlacitko MODE
              call PressUp        ;kliknuti na tlacitko UP
              call PressPtt       ;kliknuti na tlactiko PTT
              return              ;navrat
                                  
ServCtcssUp   call PressMode      ;kliknuti na tlacitko MODE
              call PressMode      ;kliknuti na tlacitko MODE
              call PressUp        ;kliknuti na tlactiko UP
              call PressPtt       ;kliknuti na PTT
              return              ;navrat
                                  
ServStream    clrf straddr        ;adresa je 0
              movlw 255           ;255 do W
              movwf temp2         ;Wco do temp2
ServStream1   btfsc PORTA,std     ;testSTD
              goto ServStream2    ;neni
              call Delay100ms     ;je - delay 100ms
              decfsz temp2,f      ;odecte 1 z temp2
              goto ServStream1    ;neni to jeste nula
              return              ;navrat 
ServStream2   call MT2No1         ;prevede vstup z MT8870 do no1
ServStream3   clrwdt              ;smaze watchdog, proti uchylum, kteri by drzeni ton vice jak 2.3sec
              btfsc PORTA,std     ;je signal STD na log.1?
              goto ServStream3    ;ano je, skok na ServStream3
              call Beep7          ;neni, loklani pip
              movfw no1           ;no1 je do W
              bsf STATUS,RP0      ;bank1      
          		movwf EEDATA        ;Wcko do EEDATA
              bcf STATUS,RP0      ;bank0      
              movfw straddr       ;adresa do Wcka
              bsf STATUS,RP0      ;bank1      
          		movwf EEADR         ;Wcko do EEADR
          		call ProgEE         ;zapis do EE, do banky 0 prepne ProgEE sam
              movlw 255           ;255 do Wcka
              movwf temp2         ;Wcko do temp2
              incf straddr        ;inkrementace adresy
              movlw 11            ;test na 11znak - konec 10 znakoveho streamu
              subwf straddr,W     ;odecte straddr od W, vysledek do W
              btfss STATUS,Z      ;je straddr=11?
              goto ServStream1    ;ne neni, nacte se dalsi znak
              return              ;navrat








; ########################################################################
; POMOCNE VECI, TLACITKA, DELAY, BEEP, ZAKEMPENI, MT2NO1...
; ########################################################################

; Mackani virtualnich tlacitek
PressMode     call Delay100ms     ;pocka 100ms
              bsf PORTA, modekey  ;stiskne tlacitko MODE
              call Delay100ms     ;pocka 100ms
              bcf PORTA, modekey  ;uvolni tlacitko MODE
              call Delay100ms     ;pocka 100ms
              return              ;navrat

PressUp       call Delay100ms     ;pocka 100ms
              bsf PORTA, upkey    ;stiskne tlacitko UP
              call Delay100ms     ;pocka 100ms
              bcf PORTA, upkey    ;uvolni tlacitko UP
              call Delay100ms     ;pocka 100ms
              return              ;navrat

PressPtt      call Delay100ms     ;pocka 100ms
              bsf PORTB, ptt      ;zaklicuje
              call Delay300ms     ;pocka 300ms
              bcf PORTB, ptt      ;odklicuje
              call Delay100ms     ;pocka 100ms
              return              ;navrat


; nacte hodnotu z MT8870, prevrati bity a ulozi do no1
MT2No1        clrf no1            ;smaze no1
              btfsc PORTB,3       ;je bit PORTB,3 log.1?
              bsf no1,0           ;ano je, nastavi bit 0 reg no1 do log.1
              btfsc PORTB,2       ;je bit PORTB,2 log.1?
              bsf no1,1           ;ano je, nastavi bit 1 reg no1 do log.1
              btfsc PORTB,1       ;je bit PORTB,1 log.1?
              bsf no1,2           ;ano je, nastavi bit 2 reg no1 do log.1
              btfsc PORTB,0       ;je bit PORTB,0 log.1?
              bsf no1,3           ;ano je, nastavi bit 3 reg no1 do log.1
              return              ;navrat
              

; Generovani zvuku - generuje to tony a melodie
Beep1         movlw 100           ;delka
              movwf temp
              movlw 100           ;ton
              goto BeepLoop       ;vygenerovani tonu
Beep2         movlw 250           ;delka
              movwf temp
              movlw 100           ;ton
              goto BeepLoop       ;vygenerovani tonu
Beep3         movlw 150           ;delka
              movwf temp
              movlw 75            ;ton
              goto BeepLoop       ;vygenerovani tonu
Beep4         movlw 120           ;delka
              movwf temp
              movlw 80            ;ton
              goto BeepLoop       ;vygenerovani tonu
Beep5         movlw 80            ;delka
              movwf temp
              movlw 200           ;ton
              goto BeepLoop       ;vygenerovani tonu
Beep6         movlw 255           ;delka
              movwf temp
              movlw 200           ;ton
              goto BeepLoop       ;vygenerovani tonu
Beep7         movlw 250           ;delka
              movwf temp
              movlw 30            ;ton
              goto BeepLoop       ;vygenerovani tonu
Beep8         movlw 50            ;delka
              movwf temp
              movlw 100           ;ton
              goto BeepLoop       ;vygenerovani tonu
Beep9         movlw 150           ;delka
              movwf temp
              movlw 100           ;ton
              goto BeepLoop       ;vygenerovani tonu
BeepLoop      bsf PORTB,beep      ;pin beep do log.1
              call BeepDel        ;pauza podle obsahu W
              bcf PORTB,beep      ;pin beet do log.0
              call BeepDel        ;pauza podle obsahu W
              decfsz temp,f       ;dekrementace temp, preskok temp, pokud je 0
              goto BeepLoop       ;skok na BeepLoop
              return              ;navrat

BeepRepeater  movlw 3             ;pocet opakovani = 3 do W
              movwf temp2         ;W do temp2
BeepRepeater1 call Beep1          ;beep
              call Beep3
              call Beep4
              decfsz temp2,f      ;dekrementace temp2 a preskok, pokud temp2=0
              goto BeepRepeater1  ;skok na BeepRepeater
              return              ;navrat

BeepDel		    movwf pau           ;W do pau
BeepDel1      clrwdt              ;smaze watchdog
              nop                 ;nic
              nop                 ;nic
              decfsz pau,f        ;dekrementace pau a preskok, pokud pau=0
		          goto BeepDel1       ;skok na BeepDel1
		          return              ;navrat
  

  
; Generovani casovych prodlev pro ruzne veci
Delay1Sec 	  movlw 0FFh          ;255 do W
		          movwf count         ;W do count
DelayLoop	    call Pause          ;pauza
		          call Pause
		          call Pause
		          call Pause
		          call Pause
		          decfsz count,f      ;dekrementace count a preskok, pokud count=0
		          goto DelayLoop      ;skok na DelayLoop
		          return              ;navrat
             
Delay300ms    call Delay100ms     ;pocka 100ms
              call Delay100ms 
              call Delay100ms 
              return              ;navrat

Delay100ms	  movlw 25            ;25 do W
		          movwf count         ;W do count
DelayL100	    call Pause          ;pauza
		          call Pause
		          call Pause
		          call Pause
		          call Pause
		          decfsz count,f      ;dekrementace count a preskok, pokud count=0
		          goto DelayL100      ;skok na DelayL100
		          return              ;navrat

Pause 		    movlw 0FFh          ;255 do W
		          movwf pau           ;W do pau
PauseLoop	    clrwdt              ;smaze watchdog
              decfsz pau,f        ;dekrementace pau a preskok, pokud pau=0
		          goto PauseLoop      ;skok na PauseLoop
		          return              ;navrat


; Zakempuje na domaci kanal a off CTCSS
Zakempuj      movlw domkanal      ;konstanta domkanal do W
              call ChannelSet     ;nastavi kanal podle obsahu W
              movlw 0             ;0 do W, 0 znamena vypnute ctcss
              call CtcssSet       ;nastavi ctcss podle obsahu W
              return              ;navrat





; ########################################################################
; PRACE S EEPROM...
; ########################################################################

; Cteni z EEPROM. Vstupem je adresa ve W a vystupem jsou data ve W
ReadEE		    bsf STATUS,RP0       ;bank 1
              movwf EEADR          ;W do EEADR
		          bsf EECON1,RD        ;zahajeni cteni
ReadEE1	      btfsc EECON1,RD      ;je RD bit log.1?
		          goto ReadEE1         ;neni
              movfw EEDATA         ;EEDATA do W
		          bcf STATUS,RP0       ;bank 0
		          return               ;navrat


; Zapis do EEPROM. Vstupem je nastavena bank1 a naplnene EEDATA a EEADR
ProgEE		    bsf EECON1,WREN      ;povoleni zapisu
      		    movlw 55h            ;povinna sekvence
      		    movwf EECON2
      		    movlw 0AAh
      		    movwf EECON2
      		    bsf EECON1,WR        ;zapis do EE
ProgEE1	      btfsc EECON1,WR      ;je WR bit log.1?
      		    goto ProgEE1         ;neni
      	 	    bcf STATUS,RP0       ;bank 0
      		    return               ;navrat






; ########################################################################
; SKAKACKA PO KANALECH A CTCSS KODECH
; ########################################################################
; Skakani po kanalech. Vstup je cislo kanalu ve W v intervalu <1;8>
; Behem teto rutiny nesmi dojit k vypadku elektriky, jinak je nutna korekce 
; v servisnim menu, ale je to dost mala pravdepodobnost.
ChannelSet    movwf chci          ;Wcko do chci
              call Delay1Sec      ;pocka 1 sec
              call PressMode      ;klikne na tlacitko MODE
              call Beep7          ;lokalni beep
              movlw adrakt        ;adresa EE s aktualnim kanalem do W
              call ReadEE         ;precteni EE
              movwf akt           ;vysledek z W do akt              
ChannelSet1   movfw akt           ;hodnota z akt do W
              subwf chci,w        ;odecteni chci od W a vyddledek do W
              btfsc STATUS,Z      ;je vysledek O? Tedy je kanal co chci shody s aktualnim?
              goto ChanellSet2    ;ano -> ukonceni
              call PressUp        ;klikne na tlacitko UP
              incf akt            ;ne, inkrementace aktualniho
              movlw 9             ;maximalne kanalu +1, tedy 9 do W
              subwf akt,w         ;odecteni 9 od W, vysledek do W
              btfss STATUS,Z      ;je v akt cislo 9?
              goto ChannelSet1    ;ne neni
              movlw 1             ;ano je a proto do nej dam 1
              movwf akt           ;ano je a proto do nej dam 1
              goto ChannelSet1    ;uzavrena smycka na ChannelSet1
ChanellSet2   movfw akt           ;akt do Wcka  
              bsf STATUS,RP0      ;bank1      
          		movwf EEDATA        ;Wcko do EEDATA
              bcf STATUS,RP0      ;bank0      
              movlw adrakt        ;adresa do Wcka
              bsf STATUS,RP0      ;bank1      
          		movwf EEADR         ;Wcko do EEADR
          		call ProgEE         ;zapis do EE, do banky 0 prepne ProgEE sam
              call PressPtt       ;klikne na tlacitko PTT
              return              ;navrat


; Skakani po CTCSS kodech. Vstup je cislo kodu ve W
; Behem teto rutiny nesmi dojit k vypadku elektriky, jinak je nutna korekce 
; v servisnim menu, ale je to dost mala pravdepodobnost.
CtcssSet      movwf chci          ;Wcko do chci
CtcssSetT2    call Delay1Sec      ;pocka 1 sec
              call PressMode      ;klikne na tlacitko MODE
              call PressMode      ;klikne na tlacitko MODE
              movlw adrctcss      ;adresa EE s aktualnim kanalem do W
              call ReadEE         ;precteni EE
              movwf akt           ;vysledek z W do akt              
CtcssSet1     movfw akt           ;hodnota z akt do W
              subwf chci,w        ;odecteni chci od W a vyddledek do W
              btfsc STATUS,Z      ;je vysledek O? Tedy je kanal co chci shody s aktualnim?
              goto CtcssSet2      ;ano -> ukonceni
              call PressUp        ;klikne na tlacitko UP
              incf akt            ;ne, inkrementace aktualniho
              movlw 39            ;maximalne kanalu +1, tedy 39 do W
              subwf akt,w         ;odecteni 39 od W, vysledek do W
              btfss STATUS,Z      ;je v akt cislo 39?
              goto CtcssSet1      ;ne neni
              movlw 0             ;ano je a proto do nej dam 0
              movwf akt           ;ano je a proto do nej dam 0
              goto CtcssSet1      ;uzavrena smycka na CtcssSet1
CtcssSet2     movfw akt           ;akt do Wcka  
              bsf STATUS,RP0      ;bank1      
          		movwf EEDATA        ;Wcko do EEDATA
              bcf STATUS,RP0      ;bank0      
              movlw adrctcss      ;adresa do Wcka
              bsf STATUS,RP0      ;bank1      
          		movwf EEADR         ;Wcko do EEADR
          		call ProgEE         ;zapis do EE, do banky 0 prepne ProgEE sam
              call PressPtt       ;klikne na tlacitko PTT
              return              ;navrat




         
 end

